package java.io;

import java.nio.CharBuffer;

public abstract class Reader implements Readable, Closeable
{
    protected Object lock;
    private static final int maxSkipBufferSize = 8192;
    private char[] skipBuffer;
    
    protected Reader() {
        this.skipBuffer = null;
        this.lock = this;
    }
    
    protected Reader(Object lock) {
        this.skipBuffer = null;
        if (lock == null) {
            throw new NullPointerException();
        }
        this.lock = lock;
        return;
    }
    
    @Override
    public int read(CharBuffer charBuffer) throws IOException {
        int remaining = charBuffer.remaining();
        char[] src = new char[remaining];
        int read = this.read(src, 0, remaining);
        if (read > 0) {
            charBuffer.put(src, 0, read);
        }
        return read;
    }
    
    public int read() throws IOException {
        char[] array = { '\0' };
        if (this.read(array, 0, 1) == -1) {
            return -1;
        }
        return array[0];
    }
    
    public int read(char[] array) throws IOException {
        return this.read(array, 0, array.length);
    }
    
    public abstract int read(char[] p0, int p1, int p2) throws IOException;
    
    public long skip(long n) throws IOException {
        if (n < 0L) {
            throw new IllegalArgumentException("skip value is negative");
        }
        int n2 = (int)Math.min(n, 8192L);
        synchronized (this.lock) {
            if (this.skipBuffer == null || this.skipBuffer.length < n2) {
                this.skipBuffer = new char[n2];
            }
            while (n > 0L) {
                int read = this.read(this.skipBuffer, 0, (int)Math.min(n, (long)n2));
                if (read == -1) {
                    break;
                }
                n -= (long)read;
            }
            return n - n;
        }
    }
    
    public boolean ready() throws IOException {
        return false;
    }
    
    public boolean markSupported() {
        return false;
    }
    
    public void mark(int n) throws IOException {
        throw new IOException("mark() not supported");
    }
    
    public void reset() throws IOException {
        throw new IOException("reset() not supported");
    }
    
    @Override
    public abstract void close() throws IOException;
}
